# NOTE: If you're running this script, make sure to set your working directory first with setwd("Path/to/this/directory")

##---------------------------------------------------------------##
##                Load libraries and import data                 ##
##---------------------------------------------------------------##

# Load libraries
library(readxl)
library(writexl)
library(ggplot2)
library(dplyr)
library(data.table)
library(extrafont)

# Import the dataset
all_threats <- read_excel("input/threats.xlsx", sheet = "Threats (2.)")
all_papers <- read_excel("input/threats.xlsx", sheet = "Papers (Short)")


##---------------------------------------------------------------##
##                       Threat Overview                         ##
##---------------------------------------------------------------##
##---------------------------------------------------------------##
##                       Helper Functions                        ##
##---------------------------------------------------------------##

# Returns a string with a concatenation of all unique values in the given column
list_all_values <- function(dataframe, column_name, separator="\n") {
  value_list <- ""
  for (value in unique(dataframe[[column_name]])) {
    if (!is.na(value)) {
      if (value_list == "") { # If list is empty, to prevent seperator at spot 1
        value_list <- value
      } else{
        value_list <- paste(value_list, value, sep=separator)
      }
    }
  }
  if (value_list == "") {
    value_list = NA
  }
  return(value_list)
}

# Counts and groups threats by code. fulltext and validity type are lost
get_threat_summary <- function(threats_dataframe) {
  summary_df <- read.csv(text="Count,Threat,Theme,Category,Mitigation,Evidence,Refs")
  threats <- unique(threats_dataframe$Threat)
  for (threat_code in threats) {
    threat_occurences <- threats_dataframe[threats_dataframe$Threat == threat_code, ]
    threat_count <- nrow(threat_occurences)
    threat_theme <- threat_occurences$Theme[1]
    threat_category <- threat_occurences$Category[1]
    mitigation_techniques <- list_all_values(threat_occurences, "Mitigation")
    evidence <- list_all_values(threat_occurences, "Evidence")
    refs <- list_all_values(threat_occurences, "Ref", ",")
    summary_df[nrow(summary_df) + 1, ] <- c(threat_count, threat_code, threat_theme, threat_category, mitigation_techniques, evidence, refs)
  }
  return(summary_df)
}

# Counts and groups threats by category. fulltext, threat code and validity type are lost
get_category_summary <- function(threats_dataframe) {
  summary_df <- read.csv(text="Count,Theme,Category,Mitigation,Evidence,Refs,Refcount")
  categories <- unique(threats_dataframe$Category)
  for (category in categories) {
    threat_occurences <- threats_dataframe[threats_dataframe$Category == category, ]
    threat_count <- nrow(threat_occurences)
    threat_theme <- threat_occurences$Theme[1]
    threat_category <- threat_occurences$Category[1]
    mitigation_techniques <- list_all_values(threat_occurences, "Mitigation")
    evidence <- list_all_values(threat_occurences, "Evidence")
    refs <- list_all_values(threat_occurences, "Ref", ",")
    ref_count <- length(unique(threat_occurences$Ref))
    summary_df[nrow(summary_df) + 1, ] <- c(threat_count, threat_theme, category, mitigation_techniques, evidence, refs, ref_count)
  }
  return(summary_df)
}

# Counts and groups threats by themes. fulltext, threat code, validity type and category are lost
get_theme_summary <- function(threats_dataframe) {
  summary_df <- read.csv(text="Count,Theme,Refs,Refcount")
  categories <- unique(threats_dataframe$Theme)
  for (category in categories) {
    threat_occurences <- threats_dataframe[threats_dataframe$Theme == category, ]
    threat_count <- nrow(threat_occurences)
    threat_theme <- threat_occurences$Theme[1]
    refs <- list_all_values(threat_occurences, "Ref", ",")
    ref_count <- length(unique(threat_occurences$Ref))
    summary_df[nrow(summary_df) + 1, ] <- c(threat_count, threat_theme, refs, ref_count)
  }
  return(summary_df)
}

# Print short summary for each unique theme
print_theme_summary <- function(theme_dataframe) {
  writeLines(paste("Summary for theme: ", theme_dataframe[1,]$Theme, "\n",
                   "- Number of threats: ", nrow(theme_dataframe), "\n"))
}


# Print summary for the selected threat in the given dataframe
print_threat_summary <- function(threat_code, threats_dataframe) {
  threat_occurences <- threats_dataframe[threats_dataframe$Threat == threat_code, ]
  mitigation_techniques <- list_all_values(threat_occurences, "Mitigation")
  evidence <- list_all_values(threat_occurences, "Evidence")
  writeLines(paste("Summary for threat: ", threat_code, "\n", 
                   "- Number of Occurences: ", nrow(threat_occurences), "\n",
                   "Mitigation Techniques: ", mitigation_techniques, "\n",
                   "Evidence: ", evidence))
}

replace_char_with_logical <- function(column) {
  # Replace yes and no with logical
  # only no means no
  column <- case_when(
    column == "yes" ~ TRUE,
    column == "no" ~ FALSE,
    column == "limitations" ~ TRUE,
    column == "other" ~ TRUE,
  )
  return(column)
}

replace_char_with_logical_hard <- function(column) {
  # Replace yes and no with logical
  # only yes means yes
  column <- case_when(
    column == "yes" ~ TRUE,
    column == "no" ~ FALSE,
    column == "limitations" ~ FALSE,
    column == "other" ~ FALSE,
  )
  return(column)
}

get_paper_summary <- function(all_papers) {
  n_papers <- nrow(all_papers)
  years_summary <- summary(all_papers$year)
  ttv_summary <- summary(replace_char_with_logical(all_papers$ttv))
  ttv_section_summary <- summary(replace_char_with_logical(all_papers$`ttv section`))
  ttv_types_summary <- summary(replace_char_with_logical(all_papers$`ttv types`))
  ttv_section_summary_hard <- summary(replace_char_with_logical_hard(all_papers$`ttv section`))
  ttv_types_summary_hard <- summary(replace_char_with_logical_hard(all_papers$`ttv types`))  
  
  summary_list = list(Number_of_Papers = n_papers, Years_Summary = years_summary, TTV_Summary = ttv_summary,
                      TTV_Section_Summary = ttv_section_summary, TTV_Section_Summary_Hard = ttv_section_summary_hard,
                      TTV_Types_Summary = ttv_types_summary, TTV_Types_Summary_Hard = ttv_types_summary_hard)
  return(summary_list)
}

print_paper_summary <- function(paper_summary) {
  print(paste0("Number of Papers: ", paper_summary$Number_of_Papers))
  print(paste0("Papers reporting Threats: ", paper_summary$TTV_Summary[3],
               " (", round(as.numeric(unname(paper_summary$TTV_Summary[3])) / paper_summary$Number_of_Papers, digits=2) * 100, " %)"))
  print(paste0("Papers with TTV Section: ", paper_summary$TTV_Section_Summary_Hard[3],
               " (", round(as.numeric(unname(paper_summary$TTV_Section_Summary_Hard[3])) / paper_summary$Number_of_Papers, digits=2) * 100, " %)"))
  print(paste0("Papers with TTV Types: ", paper_summary$TTV_Types_Summary_Hard[3],
               " (", round(as.numeric(unname(paper_summary$TTV_Types_Summary_Hard[3])) / paper_summary$Number_of_Papers, digits=2) * 100, " %)"))
  
}

get_paper_summary_per_year <- function(all_papers) {
  ttv_per_year <- data.frame(Year = all_papers$year, TTV = replace_char_with_logical(all_papers$`ttv section`))
  ttv_count_true = vector()
  ttv_count_false = vector()
  for (year in 1979:2019) {
    count_true = nrow(ttv_per_year[ttv_per_year$Year==year & ttv_per_year$TTV==TRUE,])
    count_false = nrow(ttv_per_year[ttv_per_year$Year==year & ttv_per_year$TTV==FALSE,])
    ttv_count_true <- append(ttv_count_true, count_true)
    ttv_count_false <- append(ttv_count_false, count_false)
  }
  ttv_per_year_total <- data.frame(Year = 1979:2019, TTV_Count_True = ttv_count_true, TTV_Count_False = ttv_count_false)
  return(ttv_per_year_total)
}

plot_ttv_per_year <- function(ttv_per_year_total, palette){
  pdf(file = 'output/plots/plot_ttv_per_year.pdf', width=12, height=4, family = "Times")
  par(oma=c(0,0,0,0))
  par(mar=c(3,2,0,0))
  t_ttv_per_year_total <- t(ttv_per_year_total)
  colnames(t_ttv_per_year_total) <- t_ttv_per_year_total[1,]
  t_ttv_per_year_total <- t_ttv_per_year_total[2:3,]
  rownames(t_ttv_per_year_total) <- c("TTV Reported", "No TTV Reported")
  t_ttv_per_year_total <- data.matrix(t_ttv_per_year_total)
  bar <- barplot(t_ttv_per_year_total, 
           col=palette,
           border="white",
           space=0.04,
           # font.axis=2,
           cex.names=1.2,
           ylim=c(0,21),
           )
  text(bar, ttv_per_year_total$TTV_Count_True+ttv_per_year_total$TTV_Count_False+0.5, ttv_per_year_total$TTV_Count_True+ttv_per_year_total$TTV_Count_False, 
       cex=.8, col="#666666")
  legend(x=-0.5,y=20.75, legend = c("Published papers with a \"threats to validity\" section","Published papers without a \"threats to validity\" section" ) , 
         col = c("#728dc4", "#d66863") , 
         bty = "n", pch=20 , pt.cex = 1, cex = 1.2, horiz = FALSE, inset = c(0.05, 0.05), y.intersp=1.5)
  dev.off()
}


##---------------------------------------------------------------##
##                         Main Script                           ##
##---------------------------------------------------------------##
##----------------------  Set Variables  ------------------------##

theme_summary <- get_theme_summary(all_threats)
threat_summary <- get_threat_summary(all_threats)
category_summary <- get_category_summary(all_threats)

# Get unique themes, categories and codes
themes <- unique(all_threats$Theme)
categories <- unique(all_threats$Category)
threats <- unique(all_threats$Threat)

blue_pink  <- c("#00D1E4", "#ff355e")
blue_red   <- c("#728dc4", "#d66863")

##--------------------  Execute Functions  ----------------------#

# Export summaries
write_xlsx(theme_summary, "output/theme_summary.xlsx")
# print_threat_summary("Low number of participants", all_threats)
write_xlsx(threat_summary, "output/threat_summary.xlsx")
write_xlsx(category_summary, "output/category_summary.xlsx")



# Categories
category_summary <- transform(category_summary, Count = as.numeric(Count))
category_summary <- category_summary[order(-category_summary$Count),]
category_summary$Category <- factor(category_summary$Category, levels = category_summary$Category)
category_data <- data.frame(
  x=category_summary$Category,
  y=category_summary$Count
)

# Threats
threat_summary <- transform(threat_summary, Count = as.numeric(Count))
threat_summary <- threat_summary[order(-threat_summary$Count),]
threat_summary$Threat <- factor(threat_summary$Threat, levels = threat_summary$Threat)
threat_data <- data.frame(
  x=threat_summary$Threat,
  y=threat_summary$Count
)


##--------------------------  Plots  ----------------------------##

ggplot(category_data, aes(x=x, y=y)) +
  geom_segment( aes(x=x, xend=x, y=0, yend=y), color="grey") +
  geom_point( color="orange", size=4) +
  coord_flip() +
  theme_light() +
  theme(
    panel.grid.major.x = element_blank(),
    panel.border = element_blank(),
    axis.ticks.x = element_blank(),
    # axis.text.x=element_text(angle = 90, vjust = 0.5, hjust=1)
  ) +
  xlab("Threat Category") +
  ylab("Number of Occurrences")

ggplot(threat_data, aes(x=x, y=y)) +
  geom_segment( aes(x=x, xend=x, y=0, yend=y), color="grey") +
  geom_point( color="orange", size=4) +
  theme_light() +
  theme(
    panel.grid.major.x = element_blank(),
    panel.border = element_blank(),
    axis.ticks.x = element_blank(),
    axis.text.x=element_blank()
  ) +
  xlab("Threat") +
  ylab("Number of Occurrences")

##---------------------------------------------------------------##
##                       Paper Overview                          ##
##---------------------------------------------------------------##

summary(all_papers)
paper_summary <- get_paper_summary(all_papers)
print_paper_summary(paper_summary)
ttv_per_year_total <- get_paper_summary_per_year(all_papers)
plot_ttv_per_year(ttv_per_year_total, blue_red)

