# NOTE: If you're running this script, make sure to set your working directory first with setwd("Path/to/this/directory")
# This script is intended to be ran after the "threat-review.R". Make sure that the threat_summary.xlsx and category_summary.xlsx files are present in the output folder.

##---------------------------------------------------------------##
##                Load libraries and import data                 ##
##---------------------------------------------------------------##

# Load libraries
library(readxl)
library(writexl)
library(ggplot2)
library(hrbrthemes)
library(dplyr)
library(Hmisc)

# Import the dataset
results_1 <- read_excel("input/evidence.xlsx", sheet = "1 Programming Experience")
results_2 <- read_excel("input/evidence.xlsx", sheet = "2 Program Length")
results_3 <- read_excel("input/evidence.xlsx", sheet = "3 Comprehension Measures")
all_threats <- read_excel("input/threats.xlsx", sheet = "Threats (2.)")
threat_summary <- read_excel("output/threat_summary.xlsx")
category_summary <- read_excel("output/category_summary.xlsx")

##---------------------------------------------------------------##
##                       Helper Functions                        ##
##---------------------------------------------------------------##
4
get_count_array <- function(count_these_values, count_in_here){
  Count = vector()
  for (value in count_these_values) {
    value_count <- sum(count_in_here == value, na.rm=TRUE)
    Count <- append(Count, value_count)
  }
  return(Count)
}

get_evidence_profile <- function(results) {
  Type <- c(5, 4, 3, 2, 1, -1, -2, -3, -4, -5)
  Count <- get_count_array(Type, as.numeric(results$`Final Score`))
  evidence_profile <- data.frame(Type, Count)
  return(evidence_profile)
}

print_profile_results <- function(evidence_profile) {
  print(paste0("Total Number of Evidence:", sum(evidence_profile$Count)))
}

get_bar_label <- function(profile_count_list) {
  entry_index = 1
  for (entry in profile_count_list) {
    if (profile_count_list[entry_index] == 0) {
      profile_count_list[entry_index] = ""
    }
    entry_index = entry_index + 1
  }
  return(profile_count_list)
}

generate_ep_bar_plot <- function(profile, palette, title, filename) {
  # profile <- profile %>% 
  #   mutate(mycolor = ifelse(Type>0, "type2", "type1"))
  
  # number_ticks <- function(n) {function(limits) pretty(limits, n)}
  
  # switch_second_half <- c(1,1,1,1,1,-1,-1,-1,-1,-1)
  # profile$Count <- profile$Count * switch_second_half
  pdf(file = paste0('output/plots/', filename, ".pdf"), width=7, height=5)
  par(oma=c(0,0,0,0))
  par(mar=c(4.5,3.5,2,0))
  bar <- barplot( height = profile$Count,
                  names.arg = profile$Type,
                  las = 1,
                  col = palette,
                  border = NA,  # eliminates borders around the bars
                  # main = title,
                  xlab = "Score",
                  ylab = "",
                  font.lab=2,
                  font.axis=2,
                  ylim=c(0,20),
                  cex.lab= 1.25,
                  cex.axis= 1.25,
                  cex.names=1.25,
                  yaxt="n"
                  # cex.title = 1.25
  )
  axis(2, 
       at = c(0, 5, 10, 15, 20),
       labels=c("","","", "", "")
       )
  # minor.tick(ny = 5, nx=0,  # Ticks density
  #            tick.ratio = 0.5)
  title(ylab="Count", cex.lab=1.25, cex.axis=1.25, font.lab=2, mgp=c(1.5,0,0))
  text(bar, profile$Count+0.6, get_bar_label(profile$Count), cex=1.25)
  type_index = 1
  for (evidence_count in profile$Count) {
    if (evidence_count == 0) {
      
    } else {
      for (line_index in 1:evidence_count){
        rect(bar[type_index]-0.5, line_index, bar[type_index]+0.5 , line_index, border="#FFFFFF")
      }
    }
    type_index <- type_index + 1
  }
  legend(x=6.5,y=20.75, legend = c("Supporting Evidence","Contradicting Evidence" ) , 
        col = c("#728dc4", "#d66863") , 
        bty = "n", pch=20 , pt.cex = 2, cex = 1.25, horiz = FALSE, inset = c(0.05, 0.05), y.intersp=1.5)
  # axis(1, main = "xaxt = 'n'")
  
  dev.off()
  return(bar)
}

##---------------------------------------------------------------##
##                         Main Script                           ##
##---------------------------------------------------------------##

##----------------------  Set Variables  ------------------------##

evidence_profile_1 <- get_evidence_profile(results_1)
evidence_profile_2 <- get_evidence_profile(results_2)
evidence_profile_3 <- get_evidence_profile(results_3)

blue_pink         <- c("#005fe4","#005fe4","#005fe4","#005fe4","#005fe4",
                       "#ff355e","#ff355e","#ff355e","#ff355e","#ff355e")
blue_pink_palette <- c("#00D1E4","#7fe8f1","#e5fafc","#b2f1f6","#e5fafc",
                       "#ffd6de","#ffaebe","#ffd6de","#ff859e","#ff355e")
blue_red_palette <- c("#728dc4","#8ea3cf","#aabadb","#c6d1e7","#e2e8f3",
                      "#f6e0df", "#eec2c0", "#e6a4a1", "#de8682", "#d66863")
blue_red_palette_alt <- c("#728dc4","#aabadb","#c6d1e7","#c6d1e7","#e2e8f3",
                          "#f6e0df", "#eec2c0", "#eec2c0", "#e6a4a1", "#d66863")

##-----------------------  Create Plots  ------------------------##
title_ep_1 = "Evidence Profile 1: Programming Experience"
filename_1 = "evidence-profile-1"
title_ep_2 = "Evidence Profile 2: Program Length"
filename_2 = "evidence-profile-2"
title_ep_3 = "Evidence Profile 3: Comprehension Measures"
filename_3 = "evidence-profile-3"
generate_ep_bar_plot(evidence_profile_1, blue_red_palette, title_ep_1, filename_1)
generate_ep_bar_plot(evidence_profile_2, blue_red_palette, title_ep_2, filename_2)
generate_ep_bar_plot(evidence_profile_3, blue_red_palette, title_ep_3, filename_3)

##---------------------  Print Summaries  -----------------------##
# summary(results_1_experience)
# summary(results_2_length)
# summary(results_3_measures)
# summary(all_threats)
# summary(threat_summary)
# summary(category_summary)

print("Evidence Profile 1: Programming Experience")
print_profile_results(evidence_profile_1)
print("Evidence Profile 2: Program Length")
print_profile_results(evidence_profile_2)
print("Evidence Profile 3: Comprehension Measures")
print_profile_results(evidence_profile_3)
